 Program

     omit('******* documentation **********')
@============================================================================
 Created: August 6, 2003  by Mark Goldberg
 Purpose: To control the launch of the Clarion IDE, to prevent the Registry.TRF being trashed

 Todo   : Link in CW IDE Icon`s to make it easy to create a short cut with the proper icon
 Todo   : Create a GUI UI
 Todo   : Handle more sophisticated cases (launching from multiple machines etc.)

 Updates: 2/1/04 Minor coding changes
@============================================================================
!end-omit('******* documentation **********')

 map
 end !an map is required to use the RTL, an empty map will include %cwroot%\libsrc\builtins.clw


 include('ctMutex.inc')
Mutex_IDESingle like(ctMutex)

lcl:WorkDir  string(260) !MaxPath
lcl:INI      string(260)
lcl:Section  String( 60)
lcl:ExeName  string(260)

lcl:Argument group,pre()
VerNum        string(  2) !values like '55','60'
Edition       string(  2) !values like 'EE','PE'
             end
lcl:MultiUser byte

 code
 lcl:Argument = upper( command('1') ) !expecting a command line like:  launchIDE 60EE


!------------------------------------ example entry in WIN.INI
 ![Clarion 6.0  Enterprise Edition]
 !bin=BIN\
 !workdir=C:\CLA\C6\BIN\
 !root=C:\CLA\C6\
 !exename=C60EE
!------------------------------------ example entry in WIN.INI -end

 case lcl:Argument.VerNum
   of ''  ; do ShowUsage

   of '60'; case lcl:Argument.Edition
              of 'EE'; lcl:Section = 'Clarion 6.0  Enterprise Edition' ; lcl:Ini = 'c60ee.ini'
              of 'PE'; lcl:Section = 'Clarion 6.0'                     ; lcl:Ini = 'c60pe.ini'
            else     ; do ShowUsage
            end

   of '55'; case lcl:Argument.Edition
              of 'EE'; lcl:Section = 'Clarion 5.5  Enterprise Edition' ; lcl:Ini = 'c55ee.ini'
              of 'PE'; lcl:Section = 'Clarion 5.5'                     ; lcl:Ini = 'c55pe.ini'
            else     ; do ShowUsage
            end

   of '50'; case lcl:Argument.Edition
              of 'EE'; lcl:Section = 'Clarion 5  Enterprise Edition'   ; lcl:Ini = 'c5ee.ini'
              of 'PE'; lcl:Section = 'Clarion 5'                       ; lcl:Ini = 'c5pe.ini'
            else     ; do ShowUsage
            end

   !----------------- I believe that there was only one version prior to 5.0 ----------------
   of '40'; lcl:Section = 'Clarion 4'                       ; lcl:INI = 'clarion4.ini'
   of '21'; lcl:Section = 'Clarion for Windows V2.1'        ; lcl:INI = 'cw21.ini'
   of '20'; lcl:Section = 'Clarion for Windows V2.0'        ; lcl:Ini = 'cw20.ini'
   of '15'; lcl:Section = 'Clarion for Windows V1.5'        ; lcl:INI = 'cw15.ini'
   of '10'; lcl:Section = 'TopSpeed: Clarion for Windows'   ; lcl:INI = 'cw.ini'
 else     ; do ShowUsage
 end

 lcl:WorkDir   = clip( getini(lcl:Section,'workdir') )  !read from %windir%\win.ini
 lcl:MultiUser = choose( upper(getini('Application','MultiUser','',clip( lcl:WorkDir ) & lcl:INI)) = 'ON')


 !I don't know what happens when the multiuser status has been altered since the another IDE was launched.
 !
 !Todo: I need to track current count of IDE's for a version, when running in MultiUser mode
 !      as the first to exit, will kill the mutex, but there might be others still in the IDE
 !
 !Todo: What happens if the multiple users are on different machines?

 !Policy: Will prevent the SIMPLE case, of MultiUser is Off, and all copies are being launched from a single machine

 !---------------- if MultiUser protections are off, then do NOT allow a 2nd launch as it can trash the Registry.TRF
 !The mutex will succeed if and only if, there is not another mutex "running" on the same machine by the same name.

 !  vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
 !  vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
 if Mutex_IDESingle.Init('CWIDE_Single' & lcl:Argument.VerNum) or lcl:MultiUser
 !  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 !  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    lcl:ExeName = getINI(lcl:Section,'exename','Missing<2,4,8>')
    if lcl:ExeName = 'Missing<2,4,8>'
       Message('Sorry, but that version of CW cannot be found in your WIN.INI','Launch IDE',ICON:Hand)
    else
       lcl:ExeName = clip(lcl:WorkDir) & lcl:ExeName
       Run( lcl:ExeName, 1) !the ,1 means WAIT until completed
    end

 else
    Message('CW IDE is already running, and MultiUser Protections are NOT on.','Launch IDE',ICON:Hand)
 end

 !Message('LaunchIDE is closing|VerNum['& lcl:VerNum &']|Edition['& lcl:Edition &']')
 return
 !Note: The mutex is automatically KILL'ed by the .Destruct

ShowUsage   Routine
  Message('LaunchIDE Usage:  LaunchIDE nnEE|'                 & |
          'Where nn = CW Version {{10,15,20,21,40,50,55,60}|' & |
          'Where EE = {{EE or PE}|'                           & |
          '|'                                                 & |
          'lcl:Edition['& lcl:Argument.Edition &']|'          & |
          'lcl:VerNum['& lcl:Argument.VerNum&']'                |
         )
  return




